/*-
 * #%L
 * Siteimprove :: AEM Plugin :: Package
 * %%
 * Copyright (C) 2018 Valtech
 * %%
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * #L%
 */
window.siteimprove = window.siteimprove || {};
window.siteimprove.aem = window.siteimprove.aem || (function (window, $, logger) {
    'use strict';
    var config = {},
        /**
         *
         * @param method the http Method to use
         * @param url the url to request
         * @param data the data used in the ajax request
         * @param contentType the content type of the payload sent to the server
         * @param headers the http headers to add to the request
         * @returns {JQuery.jqXHR}
         */
        ajax = function (method, url, data, contentType, headers) {
            var options = {
                method: method,
                url: url,
                dataType: 'json',
                cache: false
            };

            if (data) {
                options.data = data;
            }

            if (contentType) {
                options.contentType = contentType;
            }

            if (headers) {
                options.headers = headers;
            }

            return $.ajax(options);
        },
        /**
         * Adds customisations when using AEM in 'classic' aka ExtJS mode.
         */
        addClassicCustomisations = function () {
            // classic ui /siteadmin watch for changes
            if (window.location.pathname.indexOf("/siteadmin") === 0) {
                $(window).on('hashchange', function () {
                    getConfig().done(callWithConfig);
                });
            }
        },
        /**
         * Adds customisations when using AEM in 'touch ui' aka CoralUI mode.
         */
        addTouchUICustomisations = function () {
            logger.debug("Adding touch UI customizations");
            // supports navigating with tree view, column view, list view and card view
            // the issue is: when we select an item the navigation event fires
            // after this event the selection change event fires
            // this interferes with the pending request for the event from navigation
            $(document).on("foundation-collection-navigate", ".cq-siteadmin-admin-childpages", function(event) {
                if (event.target.classList.contains('granite-collection-loading-title-wrapper')) {
                    // event is fired twice in AEM card view, we discard this specific event.target
                    return;
                }

                // first navigate, afterwards selection change
                var $collection = $(".cq-siteadmin-admin-childpages.foundation-collection"),
                    pathForSiteimprove = $collection.data("foundationCollectionId");

                getConfig(pathForSiteimprove).done(callWithConfig);
            });

            $(document).on("foundation-selections-change", ".cq-siteadmin-admin-childpages", function(event) {
                var $selection = $(event.target).find(".foundation-selections-item");

                // for single selection we display the report for the selected item
                if ($selection.length === 1) {
                    getConfig($selection.data("foundationCollectionItemId")).done(callWithConfig);
                } else { // fall back to parent
                    getConfig($(event.target).data("foundationCollectionId")).done(callWithConfig);
                }
            });

        },
        addCustomisations = function () {
            addClassicCustomisations();
            addTouchUICustomisations();
        },
        addLoadHandler = function (handler) {
            logger.info('Adding load handler');
            if (window.addEventListener) {
                window.addEventListener('load', handler, false);
            } else if (window.attachEvent) {
                window.attachEvent('onload', handler);
            } else {
                window.onload = handler;
            }
        },
        getPath = function () {
            return window.location.pathname.split('?')[0] + window.location.hash.split('?')[0];
        },
        /**
         *
         * @param pathToUse optional path to use for retrieving the appropriate cloud configuration. If no path is given the current path is retrieved by calling getPath()
         * @returns {JQuery.Promise<any, any, any>}
         */
        getConfig = function (pathToUse) {
            logger.info("get config " + pathToUse);
            var d = $.Deferred(),
                path = pathToUse || getPath();

            if (window.siteimprove.config) {
                // have a global config supplied somewhere
                logger.info("Using configuration from window variable");
                config = window.siteimprove.config;
                d.resolve(config);
            } else if (config.requestPath && config.requestPath === path) {
                // no change between path in config and currentPath
                logger.info("Using cached config as there has been no path change");
                d.resolve(config);
            } else {
                logger.info("Refreshing configuration for path " + path);
                // refresh configuration
                var data = {
                    path: path
                };

                // TODO: configuration for this
                ajax('GET', '/conf/global/settings/siteimprove/jcr:content.config.json', data)
                    .done(function (configResponse) {
                        logger.info('updated configuration for path ' + path);
                        config = configResponse;
                        d.resolve(config);
                    })
                    .fail(function (error) {
                        d.reject('Error retrieving config ' + error);
                    });
            }
            return d.promise();
        },
        /**
         *
         * @param config
         * @returns {JQuery.Promise<any, any, any>}
         */
        insertOverlay = function (config) {
            var overlaySrc = config.overlaySrc,
                d = $.Deferred();
            if (!overlaySrc) {
                d.reject('Empty overlay src!');
            } else {
                logger.info('Inserting overlay ' + overlaySrc);
                var s = document.getElementsByTagName('script')[0],
                    po = document.createElement('script');
                po.type = 'text/javascript';
                po.async = true;
                po.src = overlaySrc;
                s.parentNode.insertBefore(po, s);
                d.resolve();
            }
            return d.promise();
        },
        callSiteimprove = function (operation, domain, token) {
            var _si = window._si || [];

            logger.info(
                'Calling siteimprove with operation "' + operation + '", path "' + domain + '" and token "' + token + '".');
            _si.push([operation, domain, token, function () {
                logger.info('Called siteimprove with operation "' + operation + '" and path "' + domain + '".');
            }]);
        },
        clear = function () {
            callSiteimprove('clear', '', config.token);
        },
        input = function (domainAndPath) {
            callSiteimprove('input', domainAndPath, config.token);
        },
        domain = function (domain) {
            callSiteimprove('domain', domain, config.token);
        },
        recheck = function (domainAndPath) {
            callSiteimprove('recheck', domainAndPath, config.token);
        },
        recrawl = function (domain) {
            callSiteimprove('recrawl', domain, config.token);
        },
        /**
         * @param config The siteimprove configuration object, see SiteimproveContextConfiguration.
         * @param config.externalContentPath The external path of the current object as defined by mappings assigned in the cloud
         *     config.
         */
        callWithConfig = function (config) {
            var operation = config.externalContentPath.length ? 'input' : 'domain',
                path = config.externalContentPath.length ? config.publicUrl : config.domain;
            callSiteimprove(operation, path, config.token);
        },
        siCheckHtml = function(htmlToCheck, contentCheckCallback, highlightCallback, externalPath) {
            _si.push(['onHighlight', highlightCallback]);
            getConfig()
                .done(function (config) {
                    var url = externalPath || config.externalContentPath.length ? config.publicUrl : '';
                    logger.info("running content check request for url " + url);
                    _si.push(['contentcheck', htmlToCheck, url, config.token, contentCheckCallback]);
                });
        },
        unpublishCheck = function(authorPagePath) {
            var url = '/bin/private/siteimprove/unpublishcheck.json?authorResourcePath=' + authorPagePath,
                d = $.Deferred();
            ajax('GET', url)
                .done(function (unpublishResponse) {
                    logger.info(unpublishResponse);
                    d.resolve(unpublishResponse);
                })
                .fail(function (error) {
                    logger.info('Error retrieving unpublish response for ' + url + ': ' + error);
                    d.reject({});
                });
            return d.promise();
        },
        init = function () {
            logger.info("siteimprove plugin init");
            if (window.siteimprove.initCalled || top.window.siteimprove.initCalled) {
                return;
            }
            window.siteimprove.initCalled = true;

            getConfig()
                .done(insertOverlay)
                .done(function () {
                    addLoadHandler(function () {
                        callWithConfig(config);
                        addCustomisations();
                    });
                });
        };

    // public functions
    return {
        init: init,
        getConfig: getConfig,
        // supports supplying all parameters
        callSiteimprove: callSiteimprove,
        // Page Content Context (known page/content url)
        input: input,
        // Non Content Context (Multiple sites in the same cms – unknown domain)
        domain: domain,
        // page recheck
        recheck: recheck,
        // full recrawl
        recrawl: recrawl,
        // remove ui
        clear: clear,
        siCheckHtml: siCheckHtml,
        unpublishCheck: unpublishCheck,
    };

})(window, Granite.$, window.siteimprove.logger);
window.siteimprove.aem.init();
